﻿################################################################################
########################## Error Handling 2 ####################################
################################################################################
################################################################################
################################################################################

<#
Here we're going to cover how ErrorActionPreference affects CATCH.
It only catches errors when STOP is set.  It turns the non-terminating error
into a terminating error that can be picked up by CATCH.
#>

##This is a non-terminating error.  You can tell cause it's able to be picked up by
##ErrorAction.
Get-Content K:\MyFile.txt -ErrorAction SilentlyContinue;

##The same non-terminating error with a try/catch.
try { Get-Content K:\MyFile.txt; }
catch {"We caught an error."}
##Notice that the error doesn't get handled?
##That's because catch only throws on terminating errors.
##So let's set ErrorActionPreference so we can terminate errors regardless.
$ErrorActionPreference = "Stop";
try { Get-Content K:\MyFile.txt; }
catch { "We caught an error." }

##This one doesn't get caught though.
$ErrorActionPreference = "SilentlyContinue";
try { Get-Content K:\MyFile.txt; }
catch { "We caught an error." }

##You can also do it inline.
try { Get-Content K:\MyFile.txt -ErrorAction Stop; }
catch { "We caught an error." }


##Now we need to talk about Finally.
##We actually call this a try/catch/finally block.
##Terminating error
try { Get-Content K:\MyFile.txt -ErrorAction Stop; }
catch { "We caught an error."; $Error.Exception }
finally { "We're in finally now." }
##Non-terminating error
try { Get-Content K:\MyFile.txt -ErrorAction SilentlyContinue; }
catch { "We caught an error." }
finally { "We're in finally now."; $Error.clear() }
##Notice that the Finally block gets run regardless of whether the error is caught
##or not. Finally always gets run.
##Finally is typically used to perform an action regardless of the outcome.
##So you could clear variables, or write to a log or print something to the screen.

##Again, the syntax is very simple...
Try { } #Try to do something.  This is required.
Catch { } #If it fails, catch the error so you can do something with it. This is required.
Finally{} #Perform an action after both try/catch have been evaluated. Catch doesn't need to run. This is NOT required.

##That brings up a good topic... the $Error var.  This is a built-in var that
##holds a collection of objects.  This var is instantiated new for every session
##and is empty until you have an error.  And for each error, it gets added to.
##This is why you may need to clear it in the Finally block.

asdf
ipconfi
$Error[0]



################# Exercises #################

<#
1. Create an integer variable and assign some text to it.  Then handle the error and clear the error variable after.
2. Query for a service named "MinionWare". If it doesn't exist, print "Service doesn't exist" to the screen.
#>














